function DNA_comp_scDir(method)
%-----------------------Pedro Saa 2016-------------------------------------
clc,close all
%%%%%%%%%%%%%%%%%%%%%%%% Base composition example %%%%%%%%%%%%%%%%%%%%%%%%%
rng('default');                                     % for reproducibility
p0     = [.5,.5];                                   % HMM parameters
n      = 5e2;
EMIS   = [.4,.1,.1,.4;.05,.4,.5,.05];
TRANS  = [.99,.01;.1,.9];
N      = 5e3;                                       % sampling parameters
prior1 = ones(1,2);
prior2 = ones(1,2);

% Build probability matrices
TRANS_HAT = [0,p0;zeros(size(TRANS,1),1),TRANS];
EMIS_HAT  = [zeros(1,size(EMIS,2));EMIS];
[~,observed] = hmmgenerate(n,TRANS_HAT,EMIS_HAT);   % Simulate observed seq
observed = abs(diff(observed));

% Sampler parameters
tol     = 10;
popIdx  = 1;
counter = 1;
alpha   = 50;
population(popIdx).params  = zeros(N,4);
population(popIdx).rho     = zeros(N,1);
tolerance                  = n/2;
population(popIdx).weights = ones(N,1);

%% I Rejection sampler
disp('SMC sampling in progress');
tstart = tic;
totalCount = 1;
while (counter<=N)
    
    % Sampling phase
    pi_trans1 = randg(prior1);
    pi_trans1 = pi_trans1/sum(pi_trans1);
    pi_trans2 = randg(prior2);
    pi_trans2 = pi_trans2/sum(pi_trans2);    
    TRANS    = [pi_trans1;pi_trans2];

    % Build new matrices
    TRANS_HAT = [0,p0;zeros(size(TRANS,1),1),TRANS];
    TRANS = TRANS';

    % Simulate HMM
    [~,simulated] = hmmgenerate(n,TRANS_HAT,EMIS_HAT);
    simulated     = abs(diff(simulated));
    rho           = sum(abs(simulated-observed)>0);                    % Large tolerance
    if (rho<tolerance)
        population(popIdx).params(counter,:) = [pi_trans1,pi_trans2];  % Keep both sets of parameters in the same array
        population(popIdx).rho(counter)      = rho;
        counter                              = counter+1;
    end
    totalCount = totalCount+1;
end
ESS = N;
acceptanceRate = counter/totalCount;

%% II Sequential Sampling
while (tolerance(popIdx)~=tol)      
    
    % Define tolerance and population statistics
    currTol = max([min([tolerance(popIdx)-5,prctile(rho,alpha)]),tol]);                             % Determine tolerance for next step    
	droppedParticles = find(population(popIdx).rho>currTol);
    population(popIdx).weights(droppedParticles)  = [];                                             % Remove particles below the threshold    
    population(popIdx).params(droppedParticles,:) = [];
    population(popIdx).rho(droppedParticles)      = [];
    population(popIdx).weights = population(popIdx).weights/sum(population(popIdx).weights);      	% Re-normalize previous weights
	
	% Start of next sampling step
    popIdx            = popIdx+1;																  	% Update population and tolerance
    tolerance(popIdx) = currTol;
	Nreplenish        = N-numel(population(popIdx-1).weights);										% Particles to replenish
	Nalive            = N-Nreplenish;																% Alive particles for current iteration
    
    % Compute kernel parameters for both sets of
    Kparam1 = getKernelParam(population(popIdx-1).params(:,1:2),method,population(popIdx-1).weights);
    Kparam2 = getKernelParam(population(popIdx-1).params(:,3:4),method,population(popIdx-1).weights);
	
    % Sampling loop
	counter    = 1;
	totalCount = 1;
	
    while (counter<=Nreplenish)
        totalCount = totalCount+1;
        
        % Resample from previous population
        particleIdx  = sampleParticle(1:Nalive,1,population(popIdx-1).weights); 
        prevParticle = population(popIdx-1).params(particleIdx,:);
                
        % Perform perturbation
        newTrans1 = randg(Kparam1*prevParticle(1:2)+1);
        newTrans1 = newTrans1/sum(newTrans1);
        newTrans2 = randg(Kparam2*prevParticle(3:4)+1);
        newTrans2 = newTrans2/sum(newTrans2);
        TRANSnew  = [newTrans1;newTrans2];
        if any(isnan(TRANSnew))
            continue;
        else
            if any(TRANSnew(:)==0);
                continue;
            end
        end
        
        % Build new matrices
        TRANS_HAT = [0,p0;zeros(size(TRANSnew,1),1),TRANSnew];
        TRANSnew  = TRANSnew';
        
        % Simulate HMM
        [~,simulated] = hmmgenerate(n,TRANS_HAT,EMIS_HAT);
        simulated     = abs(diff(simulated));
        rho           = sum(abs(simulated-observed)>0);
        if (rho<=currTol)
			
			% Save model parameters            
            population(popIdx).params(counter,:) = [newTrans1,newTrans2];  % Keep both sets of parameters in the same array
            population(popIdx).rho(counter,1)    = rho;
            
			% Compute weight for the accepted particle
            priorProb   = probDirichlet(prior1,newTrans1,1)*probDirichlet(prior2,newTrans2,1);
            denomWeight = population(popIdx-1).weights'*(probDirichlet(Kparam1*population(popIdx-1).params(:,1:2)+1,newTrans1,1).*...
                                                        probDirichlet(Kparam2*population(popIdx-1).params(:,3:4)+1,newTrans2,1));           
			population(popIdx).weights(counter,1) = priorProb/denomWeight;
			
			% Update particle counter
			if ~isnan(population(popIdx).weights(counter,1)) && (population(popIdx).weights(counter,1)>0)
                counter = counter+1;
            end			
        end		
    end
    
    % Normalize weights for the replenished particles
	population(popIdx).weights = population(popIdx).weights/sum(population(popIdx).weights);

	% Add previous alive particles to the current population
	population(popIdx).params  = [population(popIdx).params;population(popIdx-1).params];
	population(popIdx).rho     = [population(popIdx).rho;population(popIdx-1).rho];
	population(popIdx).weights = [Nreplenish*population(popIdx).weights;Nalive*population(popIdx-1).weights]/N;       % Normalize weights appropiately
    ESS	= [ESS,1/sum(population(popIdx).weights.^2)];                                                                 % Compute effective sample size
    acceptanceRate = [acceptanceRate,Nreplenish/totalCount];
end
tstop = toc(tstart);
clearvars -except population method tolerance ESS acceptanceRate tstop
save(['population_sc_Dir_DNA_',num2str(method),'.mat']);